#include "opencv2/highgui.hpp"
#include "opencv2/core.hpp"
#include "opencv2/imgproc.hpp"
#include <iostream>
#include <json/json.h>
#include <fstream>
#include <algorithm>
#include <map>
using namespace cv;
using namespace std;
#include <filesystem>
namespace fs = std::filesystem;

//C++17
//sudo apt install libjsoncpp-dev
//sudo ln -s /usr/include/jsoncpp/json/ /usr/include/json



//struct necessary for comparison between 2 opencv points - when using std::map
//used to sort points according to X value
struct ComparePoints
{
    bool operator () (const cv::Point& a, const cv::Point& b) const
    { 
        return (a.x < b.x) || (a.x == b.x && a.y < b.y);
    }
} CP;



class PointBase {
    public:
        vector <Point2d> points;
        vector <Point2d> doneVector;
        std::map<Point2d, string, ComparePoints> pathMap;



    //loaded points in pointbase are sorted according to X value
    int load(string path)
    {
        //std::string path = "../data/";
        for (const auto & entry : fs::directory_iterator(path))
        {
            string filePath = entry.path();
            //cout << filePath.find(".json") << std::endl;
            if (filePath.find(".json") != string::npos) 
            {
                //std::cout << filePath << std::endl;
                std::ifstream coord_file(filePath, std::ifstream::binary);
                Json::Value coords;

                coord_file >> coords;
                //cout << "x: " << coords["depthPos"]["x"].asDouble() << std::endl;
                //cout << "y: " << coords["depthPos"]["y"].asDouble() << std::endl;
                Point2d newPoint = Point2d(coords["depthPos"]["x"].asDouble() , coords["depthPos"]["y"].asDouble());
                pathMap[newPoint] = filePath;
                points.push_back(newPoint);
            }

        }

        if (points.empty()) {
            cout << "No points detected, check path \n";
            return -1;
        }
        else 
        {
            sort(points.begin(),points.end(),CP); 
            return 0;
        }
    }

    void printPoints(int which=0)
    {
        if (which==1) // prints current points vector
        {
            for(int i =0; i<points.size(); i++)
            {
                cout << "point " << i <<'\n';
                cout << "x: " << points[i].x << '\n';
                cout << "y: " << points[i].y << '\n';
            }
        }
        
        if (which==0) // prints Contents of pointBase
        {
            for (const auto& [point, path] : pathMap) 
            {
                std::cout << "path: " << path<<   "; \n";
                std::cout << "x: " << point.x  << "; \n";
                std::cout << "y: " << point.y  << "; \n";
            }
        }
        

    }

    string getPath(double x, double y)
    {
        string path = pathMap[Point2d(x,y)];
        return(path);
    }


    //shows the location of loaded points on an images
    //useful for debugging
    void showPointImg()
    {
        Mat img(1200, 360, CV_8UC3,   Scalar(0, 0, 0));
        for(int i =0; i<points.size(); i++)
        {
            circle(img, Point2d(points[i].x, (points[i].y)/10),3, Scalar(255, 0, 0), 3);
        }
        imshow("Location of Images", img);//Showing the circle//
        waitKey(0);//Waiting for Keystroke//
    }
};